#
#
#  Example demonstrating the combination of Blender and ASE
#  ========================================================
#
#
#  Start       : 2012-10-16
#  Last change : 2012-10-31
#  Author      : Clemens Barth (barth@root-1.de) 
#
#
#
#
#  This script has to be executed in a terminal with:
#
#         > python example5.py
#
#  After, push "0" on the number pad. You will then see the atoms 
#  from the camera point of view. 
#
#  Notes
#  =====
#  
#  1. You do not have to install the Atomic Blender - XYZ addon!
#
#  2. More details about the options see:
#     http://wiki.blender.org/index.php/Extensions:2.6/Py/Scripts/Import-Export/XYZ
#
# 



#  The following code is a modified version of the code (3rd one) discussed 
#  on this ASE page:
#  https://wiki.fysik.dtu.dk/ase/tutorials/md/md.html#md-tutorial



from ase.io import read
from ase.io import write


import os
from copy import deepcopy
from ase.blender import blend_show, blend_render


"Demonstrates molecular dynamics for isolated particles."

from ase.calculators.emt import EMT
from ase.cluster.cubic import FaceCenteredCubic
from ase.optimize import QuasiNewton
from ase.md.velocitydistribution import MaxwellBoltzmannDistribution, \
                                        Stationary, ZeroRotation
from ase.md.verlet import VelocityVerlet
from ase import units

# Use Asap for a huge performance increase if it is installed
useAsap = True

if useAsap:
    from asap3 import EMT
    size = 4
else:
    size = 2

# Set up a nanoparticle
atoms = FaceCenteredCubic('Cu', surfaces=[[1,0,0],[1,1,0],[1,1,1]],
                          layers=(size,size,size), vacuum=4)

# Describe the interatomic interactions with the Effective Medium Theory
atoms.set_calculator(EMT())

# Do a quick relaxation of the cluster
qn = QuasiNewton(atoms)
qn.run(0.001, 10)

# Set the momenta corresponding to T=1200K
MaxwellBoltzmannDistribution(atoms, 25000*units.kB)
Stationary(atoms) # zero linear momentum
ZeroRotation(atoms) # zero angular momentum

# We want to run MD using the VelocityVerlet algorithm.
# save trajectory.
dyn = VelocityVerlet(atoms, 5*units.fs, trajectory='ase_blender_example5.traj') 


frame_list = []

#Function to print the potential, kinetic and total energy.
def printenergy(a=atoms):    #store a reference to atoms in the definition.
    epot = a.get_potential_energy() / len(a)
    ekin = a.get_kinetic_energy() / len(a)
    frame_list.append(deepcopy(a))
    print ("Energy per atom: Epot = %.3feV  Ekin = %.3feV (T=%3.0fK)  Etot = %.3feV" %
           (epot, ekin, ekin/(1.5*units.kB), epot+ekin))
dyn.attach(printenergy, interval=10)

# Now run the dynamics
printenergy()
dyn.run(2000)


file_xyz = "ase_blender_example5.xyz"
write(file_xyz, frame_list)
blend_render(file_xyz, "ase_blender_example5.blend", "ase_blender_example5", 
             image_format="AVIJPEG",
             load_frames = True,
             frame_start = 1,
             frame_end   = 20,
             blender_close = True,
             blender_save = True)



